/* Type definitions for nondeterministic finite state machine for bison,
   copyright (C) 1984 Bob Corbett and Richard Stallman

   Permission is granted to anyone to make or distribute verbatim copies of this program
   provided that the copyright notice and this permission notice are preserved;
   and provided that the recipient is not asked to waive or limit his right to
   redistribute copies as permitted by this permission notice;
   and provided that anyone possessing an executable copy
   is granted access to copy the source code, in machine-readable form,
   in some reasonable manner.

   Permission is granted to distribute derived works or enhanced versions of
   this program under the above conditions with the additional condition
   that the entire derivative or enhanced work
   must be covered by a permission notice identical to this one.

   Anything distributed as part of a package containing portions derived
   from this program, which cannot in current practice perform its function usefully
   in the absense of what was derived directly from this program,
   is to be considered as forming, together with the latter,
   a single work derived from this program,
   which must be entirely covered by a permission notice identical to this one
   in order for distribution of the package to be permitted.

 In other words, you are welcome to use, share and improve this program.
 You are forbidden to forbid anyone else to use, share and improve
 what you give them.   Help stamp out software-hoarding!  */

/* These type definitions are used to represent a nondeterministic
   finite state machine that parses the specified grammar.
   This information is generated by the function generate_states
   in the file LR0.

Each state of the machine is described by a set of items --
particular positions in particular rules -- that are the possible
places where parsing could continue when the machine is in this state.
These symbols at these items are the allowable inputs that can follow now.

A core represents one state.  States are numbered in the number field.
When generate_states is finished, the starting state is state 0
and nstates is the number of states.  (A transition to a state
whose state number is nstates indicates termination.)  All the cores
are chained together and first_state points to the first one (state 0).

For each state there is a particular symbol which must have been the
last thing accepted to reach that state.  It is the accessing_symbol
of the core.

Each core contains a vector of nitems items which are the indices
in the ritems vector of the items that are selected in this state.

The link field is used for chaining buckets that hash states by
their itemsets.  This is for recognizing equivalent states and
combining them when the states are generated.

The two types of transitions are shifts (push the lookahead token
and read another) and reductions (combine the last n things on the
stack via a rule, replace them with the symbol that the rule derives,
and leave the lookahead token alone).  When the states are generated,
these transitions are represented in two other lists.

Each shifts structure describes the possible shift transitions out
of one state, the state whose number is in the number field.
The shifts structures are linked through next and first_shift points to them.
Each contains a vector of numbers of the states that shift transitions
can go to.  The accessing_symbol fields of those states' cores say what kind
of input leads to them.

A shift to state zero should be ignored.  Conflict resolution
deletes shifts by changing them to zero.

Each reductions structure describes the possible reductions at the state
whose number is in the number field.  The data is a list of nreds rules,
represented by their rule numbers.   first_reduction points to the list
of these structures.

There is at least one shift transition present in state zero.
It leads to a next-to-final state whose accessing_symbol is
the grammar's start symbol.  The next-to-final state has one shift
to the final state, whose accessing_symbol is zero (end of input).
The final state has one shift, which goes to the termination state
(whose number is nstates, and for which there is no core structure).
The reason for the extra state at the end is to placate the parser's
strategy of making all decisions one token ahead of its actions.  */


typedef
  struct core
    {
      struct core *next;
      struct core *link;
      short number;
      short accessing_symbol;
      short nitems;
      short items[1];
    }
  core;



typedef
  struct shifts
    {
      struct shifts *next;
      short number;
      short nshifts;
      short shifts[1];
    }
  shifts;



typedef
  struct reductions
    {
      struct reductions *next;
      short number;
      short nreds;
      short rules[1];
    }
  reductions;



extern int nstates;
extern core *first_state;
extern shifts *first_shift;
extern reductions *first_reduction;
